const getRatingByField = require("./getRatingByField");

const limits = [
  {
    '@id': '/section_quality_time_limits/59490589-b854-4fb5-8660-66cd660c58df',
    '@type': 'SectionQualityTimeLimits',
    id: '59490589-b854-4fb5-8660-66cd660c58df',
    sectionId: 'ccfd5a4a-0d00-11e6-942a-005056ac0e86',
    section: '/sections/ccfd5a4a-0d00-11e6-942a-005056ac0e86',
    mark: '5',
    minQuality: '2',
    maxQuality: '5',
    minTime: '50',
    maxTime: '100',
    createdAt: '2021-05-13T09:15:03+00:00',
    updatedAt: '2021-05-13T09:15:03+00:00',
  },
  {
    '@id': '/section_quality_time_limits/60069124-0baf-42e0-8b3b-d073b79688e3',
    '@type': 'SectionQualityTimeLimits',
    id: '60069124-0baf-42e0-8b3b-d073b79688e3',
    sectionId: 'ccfd5a4a-0d00-11e6-942a-005056ac0e86',
    section: '/sections/ccfd5a4a-0d00-11e6-942a-005056ac0e86',
    mark: '4',
    minQuality: '6',
    maxQuality: '10',
    minTime: '100',
    maxTime: '149',
    createdAt: '2021-05-13T09:15:03+00:00',
    updatedAt: '2021-05-13T09:15:03+00:00',
  },
  {
    '@id': '/section_quality_time_limits/2fc1a341-a6d9-4e24-964e-b2eb0ff0a873',
    '@type': 'SectionQualityTimeLimits',
    id: '2fc1a341-a6d9-4e24-964e-b2eb0ff0a873',
    sectionId: 'ccfd5a4a-0d00-11e6-942a-005056ac0e86',
    section: '/sections/ccfd5a4a-0d00-11e6-942a-005056ac0e86',
    mark: '3',
    minQuality: '11',
    maxQuality: '12',
    minTime: '150',
    maxTime: '199',
    createdAt: '2021-05-13T09:15:03+00:00',
    updatedAt: '2021-05-13T09:15:03+00:00',
  },
  {
    '@id': '/section_quality_time_limits/c91e8918-b350-4aee-9cb8-076f0d7a9733',
    '@type': 'SectionQualityTimeLimits',
    id: 'c91e8918-b350-4aee-9cb8-076f0d7a9733',
    sectionId: 'ccfd5a4a-0d00-11e6-942a-005056ac0e86',
    section: '/sections/ccfd5a4a-0d00-11e6-942a-005056ac0e86',
    mark: '2',
    minQuality: '13',
    maxQuality: '15',
    minTime: '200',
    maxTime: '249',
    createdAt: '2021-05-13T09:15:03+00:00',
    updatedAt: '2021-05-13T09:15:03+00:00',
  },
  {
    '@id': '/section_quality_time_limits/e2616a52-7438-41d4-97e4-2eaf22599fb9',
    '@type': 'SectionQualityTimeLimits',
    id: 'e2616a52-7438-41d4-97e4-2eaf22599fb9',
    sectionId: 'ccfd5a4a-0d00-11e6-942a-005056ac0e86',
    section: '/sections/ccfd5a4a-0d00-11e6-942a-005056ac0e86',
    mark: '1',
    minQuality: '16',
    maxQuality: '18',
    minTime: '250',
    maxTime: '300',
    createdAt: '2021-05-13T09:15:03+00:00',
    updatedAt: '2021-05-13T09:15:03+00:00',
  },
];

describe('getRatingByField', () => {
  test('more than maximum limit', () => {
    expect(
      getRatingByField({
        limits,
        value: '700',
        minNameField: 'minTime',
        maxNameField: 'maxTime',
      })
    ).toBe('1');
  });

  test('1 mark ', () => {
    expect(
      getRatingByField({
        limits,
        value: '270',
        minNameField: 'minTime',
        maxNameField: 'maxTime',
      })
    ).toBe('1');
  });

  test('3 mark', () => {
    expect(
      getRatingByField({
        limits,
        value: '150',
        minNameField: 'minTime',
        maxNameField: 'maxTime',
      })
    ).toBe('3');
  });

  test('3 mark', () => {
    expect(
      getRatingByField({
        limits,
        value: '199',
        minNameField: 'minTime',
        maxNameField: 'maxTime',
      })
    ).toBe('3');
  });

  test('5 mark', () => {
    expect(
      getRatingByField({
        limits,
        value: '55',
        minNameField: 'minTime',
        maxNameField: 'maxTime',
      })
    ).toBe('5');
  });

  test('5 on limit', () => {
    expect(
      getRatingByField({
        limits,
        value: '100',
        minNameField: 'minTime',
        maxNameField: 'maxTime',
      })
    ).toBe('5');
  });

  test('less than minimum limit', () => {
    expect(
      getRatingByField({
        limits,
        value: '0',
        minNameField: 'minTime',
        maxNameField: 'maxTime',
      })
    ).toBe('1');
  });

  test('Another field', () => {
    expect(
      getRatingByField({
        limits,
        value: '11',
        minNameField: 'minQuality',
        maxNameField: 'maxQuality',
      })
    ).toBe('3');
  });
});
