const express = require("express");
const app = express();
const bodyParser = require("body-parser");
require("dotenv").config();
const cors = require('cors')

const intensityDescription = require("./services/intensity-description");
const pest = require("./services/pest");
const status = require("./services/status");
const rating = require("./services/rating");
const task = require("./services/task");
const root = require("./services/root");
const get = require("./services/get");
const list = require("./services/list");
const flush = require("./services/flush");
const apkFlush = require("./services/apk-flush");
const checkMetric = require("./services/check-metric");
const getMetric = require("./services/get-metric");
const taskHistory = require("./services/task-history");
const trolleyHistory = require("./services/trolley-history");
const deleteService = require("./services/delete");
const chemical = require("./services/chemical");
const intensity = require("./services/intensity");
const storeTask = require("./services/store-task");
const storeEvent = require("./services/store-event");
const events = require("./services/events");
const tasks = require("./services/tasks");
const store = require("./services/store");
const setPriority = require("./services/set-priority");
const setRating = require("./services/set-rating");
const avgRating = require("./services/avg-rating");

const plantService = require("./services/plant");
const collectableService = require("./services/collectable");
const codeService = require("./services/code");
const valveService = require("./services/valve");

const worker = require("./services/worker");

app.use(cors());
app.use(express.urlencoded({ limit: '100mb', extended: false }));
app.use(express.json({ limit: '100mb' }));

app.get("/flush", flush);
app.get("/apk-flush", apkFlush);

app.get("/pest/:force", pest);
app.get("/pest/:type/:force", pest);

app.get("/chemical/:force", chemical);

app.get("/intensity-description/:force", intensityDescription);
app.get("/intensity/:force", intensity);

app.get("/check-metric/:force", checkMetric);
app.get("/get-metric/:force", getMetric);
app.get("/get-metric/:sectionId/:force", getMetric);

app.get("/task-history/:workerId/:force", taskHistory);
app.get("/trolley-history/:workerId/:force", trolleyHistory);

app.post("/store-task", storeTask);
app.post("/store-event", storeEvent);

app.get("/events/:key", events);
app.get("/tasks/:key", tasks);

app.post("/store", store);
app.get("/delete/:key", deleteService);
app.get("/get/:key", get);
app.get("/list", list);

app.get("/", root);

app.get("/status/:force", status);
app.get("/status/:workerId/:force", status);

app.get("/rating/:taskId/:write", rating);
app.get("/set-priority/:workerId/:write", setPriority);
app.get("/set-rating/:workerId/:write", setRating);
app.get("/avg-rating/:workerId/:write", avgRating);

app.get("/t/:workerId/:force", task);
app.get("/w/:id/:force", worker);

app.get("/plant/:force", plantService);
app.get("/plant/:greenhouseId/:force", plantService);

app.get("/valve/:force", valveService);
app.get("/valve/:greenhouseId/:force", valveService);

app.get("/collectable/:force", collectableService);

app.get("/code/:force", codeService);
app.get("/code/:greenhouseId/:force", codeService);

const PORT = process.env.PORT || 3000;
app.listen(PORT, () => {
  console.log(`Server listening on port ${PORT}`);
});
