const axios = require("axios");
const moment = require("moment");
const _ = require("lodash");
const getRequest = require("../helpers/request");

const AvgRatingService = async (req, res) => {
  try {
    const { workerId, write } = req.params;

    if (!workerId) {
      throw new Error("[/avg-rating] MISSING WORKER ID");
    }

    const worker = await axios({
      url: `${process.env.API_HOST}/workers/${workerId}`,
      method: "get",
      headers: {
        "X-Username": "agrocacher [GET PRIORITY WORKER]",
        Accept: "application/json",
        "Content-Type": "application/json",
      },
    }).then((res) => res.data);

    if (!worker.isActive) {
      throw new Error("[/avg-rating] INACTIVE WORKER");
    }

    const ratings = _.keyBy(
      await axios({
        url: `${process.env.API_HOST}/ratings?workerId=${workerId}&itemsPerPage=100`,
        method: "get",
        headers: {
          "X-Username": "agrocacher [GET PRIORITY RATINGS]",
          Accept: "application/json",
          "Content-Type": "application/json",
        },
      }).then((res) => res.data),
      "sectionId",
    );

    const requestTasks = _.map(ratings, (r) => {
      const params = {
        itemsPerPage: 1000,
        sectionId: r.sectionId,
        workerId,
        finishedAt: {
          after: moment(r.ratingUpdatedAt),
          before: moment(new Date()).endOf("day"),
        },
        properties: ["sectionId", "dynamicRating"],
        "dynamicRating[gt]": "0",
      };
      return getRequest("tasks", params);
    });

    const responseTasks = await Promise.all(requestTasks);
    const sectionsDynamicRating = responseTasks.map((r) => {
      const taskWithDynamicRating = _(r["hydra:member"])
        .map(({ sectionId, dynamicRating }) => ({
          sectionId,
          dynamicRating,
        })).value();
      const sectionDynamicRating = taskWithDynamicRating.reduce((acc, el) => ({
        sectionId: el.sectionId,
        dynamicRating: _.meanBy(taskWithDynamicRating, "dynamicRating"),
      }), { sectionId: null });
      return sectionDynamicRating;
    }).filter(({ sectionId }) => sectionId);

    const output = sectionsDynamicRating.map((section) => {
      const rating = _.find(ratings, ({ sectionId }) => section.sectionId === sectionId);
      return {
        sectionId: rating.sectionId,
        ratingId: rating.id,
        dynamicRating: Number((section.dynamicRating).toFixed(1)),
        reason: "n/a",
        saved: false,
      };
    });

    if (write === "1") {
      for (const out of output) {
        await axios({
          method: "patch",
          url: `${process.env.API_HOST}/ratings/${out.ratingId}`,
          headers: {
            "X-Username": "agrocacher [PATCH RATINGS]",
            "Content-Type": "application/merge-patch+json",
          },
          data: {
            dynamicRating: out.dynamicRating,
          },
        });
        out.saved = true;
      }
    }

    return res.json({
      error: false,
      name: worker.name,
      greenhouse: worker.greenhouse?.description,
      workerId: workerId || null,
      currentDate: moment().format(),
      output,
    });
  } catch (e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: `[/avg-rating error] [${e}]`,
    });
  }
};

module.exports = AvgRatingService;
