const redisClient = require("../client");
const _ = require('lodash');
const axios = require('axios');

const CheckMetricService = async (req, res) => {
  try {
    const { force } = req.params;

    const keys = await redisClient
      .scanAsync("0", "MATCH", `check-metric-*`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "CHECK_METRIC ERROR", e);

        return false;
      });

    if (keys) {
      let checkMetrics = {};

      await Promise.all(
        _.map(keys[1], async (k) => {
          const rawData = await redisClient.getAsync(k);

          if (rawData) {
            checkMetrics[k] = JSON.parse(rawData);
          }
        })
      );

      if (_.size(checkMetrics) <= 0 || force) {
        // always invalidate
        rawMetrics = await axios({
          method: "get",
          url: `${process.env.API_HOST}/check_metrics?itemsPerPage=1000&properties[]=id&properties[]=sectionId&properties[]=name&properties[]=category&properties[]=image`,
          headers: { "X-Username": "agrocacher [GET CHECK_METRICS]" },
        })
          .then((response) => response.data)
          .then((data) => data["hydra:member"]);

        const sectionIds = _.uniq(_.map(rawMetrics, (cm) => cm.sectionId));

        for (const sectionId of sectionIds) {
          const k = `check-metric-${sectionId}`;

          const sectionCheckMetrics = _.filter(
            rawMetrics,
            (cm) => cm.sectionId === sectionId
          );

          await redisClient.setAsync(k, JSON.stringify(sectionCheckMetrics));
          await redisClient.expireAsync(k, 3600);

          checkMetrics[k] = sectionCheckMetrics;
        }
      }

      console.log(`CHECK_METRICS REQUEST [${_.size(checkMetrics)}]`);

      return res.json(checkMetrics);
    } else {
      throw new Error('REQUEST FAIL');
    }
  } catch(e) {
    console.error(e);
    
    return res.status(500).send({
      error: true,
      message: "[/check-metric] error!",
    });    
  }
};

module.exports = CheckMetricService;
