const redisClient = require("../client");
const axios = require('axios');
const _ = require('lodash');

const ChemicalService = async (req, res) => {
  try {
    const { force } = req.params;

    const keys = await redisClient
      .scanAsync("0", "MATCH", `chemical-*`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "CHEMICAL ERROR", e);

        return false;
      });

    if (keys) {
      let chemicals = [];

      await Promise.all(
        _.map(keys[1], async (k) => {
          const rawData = await redisClient.getAsync(k);

          if (rawData) {
            chemicals.push(JSON.parse(rawData));
          }
        })
      );

      if (chemicals.length <= 0 || force === "1") {
        chemicals = await axios({
          method: "get",
          url: `${process.env.API_HOST}/chemicals?itemsPerPage=100`,
          headers: { "X-Username": "agrocacher [GET CHEMICALS]" },
        })
          .then((response) => response.data)
          .then((data) => data["hydra:member"]);

        for (const chemical of chemicals) {
          const k = `chemical-${chemical.id}`;

          await redisClient.setAsync(k, JSON.stringify(chemical));
          await redisClient.expireAsync(k, 3600);
        }
      }

      console.log(`CHEMICALS REQUEST [${chemicals.length}]`);

      return res.json(chemicals);
    } else {
      throw new Error('Chemicals error!');
    }
  } catch(e) {
    console.error(e);
    
    return res.status(500).send({
      error: true,
      message: "[/chemical] error!",
    });    
  }
};

module.exports = ChemicalService;
