const redisClient = require("../client");
const axios = require('axios');
const _ = require('lodash');
const moment = require('moment');

const CodeService = async (req, res) => {
  try {
    const { greenhouseId, force } = req.params;

    const keys = await redisClient
      .scanAsync("0", "MATCH", `code-*`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "CODE ERROR", e);

        return false;
      });

    if (keys) {
      let codes = [];

      await Promise.all(
        _.map(keys[1], async (k) => {
          const rawData = await redisClient.getAsync(k);

          if (rawData) {
            const data = JSON.parse(rawData);

            if (!greenhouseId || (greenhouseId && data.greenhouseId === greenhouseId)) {
              codes.push(data);
            }
          }
        })
      );

      let params = {
        greenhouseId,
        itemsPerPage: 0,
        "updatedAt[after]": moment().subtract(30, 'day').format(),
      }

      const total = await axios({
        method: "get",
        url: `${process.env.API_HOST}/code_metrics`,
        params,
        headers: { "X-Username": "agrocacher [GET CODES]" },
      })
        .then((response) => response.data)
        .then((data) => data["hydra:totalItems"]);      

      if (codes.length < total || force === "1") {
        codes = [];

        // FIXME: Partial loading might be better
        const dbCodes = await axios({
          method: "get",
          url: `${process.env.API_HOST}/code_metrics`,
          params: {
            ...params,
            itemsPerPage: 9999,
            "order[updatedAt]": "desc",
          },
          headers: { 
            "X-Username": "agrocacher [GET CODES]", 
            Accept: "application/json",
            "Content-Type": "application/json", 
          },
        })
          .then((response) => response.data);
          //.then((data) => data["hydra:member"]);

        for (const code of dbCodes) {
          const k = `code-${code.greenhouseId}-${code.id}`;

          const p = {
            id: code.id,
            code: code.code,
            collectableId: code.collectableId,
            plantId: code.plantId,
            greenhouseId: code.greenhouseId,
          }

          codes.push(p);

          await redisClient.setAsync(k, JSON.stringify(p));
          await redisClient.expireAsync(k, 3600);
        }
      }

      console.log(`CODES REQUEST [${codes.length}]`);

      return res.json(codes);
    } else {
      throw new Error('Codes error!');
    }
  } catch(e) {
    console.error(e);
    
    return res.status(500).send({
      error: true,
      message: "[/code] error!",
    });    
  }
};

module.exports = CodeService;
