const redisClient = require("../client");
const axios = require('axios');
const _ = require('lodash');

const CollectableService = async (req, res) => {
  try {
    const { force } = req.params;

    const keys = await redisClient
      .scanAsync("0", "MATCH", `collectable-*`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "COLLECTABLE ERROR", e);

        return false;
      });

    if (keys) {
      let collectables = [];

      await Promise.all(
        _.map(keys[1], async (k) => {
          const rawData = await redisClient.getAsync(k);

          if (rawData) {
            collectables.push(JSON.parse(rawData));
          }
        })
      );

      if (collectables.length <= 0 || force === "1") {
        collectables = [];

        // FIXME: Partial loading might be better
        const dbCollectables = await axios({
          method: "get",
          url: `${process.env.API_HOST}/collectables?isActive=true&itemsPerPage=10000`,
          headers: { 
            "X-Username": "agrocacher [GET COLLECTABLES]", 
            Accept: "application/json",
            "Content-Type": "application/json", 
          },
        })
          .then((response) => response.data);
          //.then((data) => data["hydra:member"]);

        for (const collectable of dbCollectables) {
          const k = `collectable-${collectable.id}`;

          const p = {
            id: collectable.id,
            name: collectable.name,
            parentId: collectable.parentId || null,
            type: collectable.type,
            isActive: collectable.isActive,
            category: collectable.category,
          }

          collectables.push(p);

          await redisClient.setAsync(k, JSON.stringify(p));
          await redisClient.expireAsync(k, 3600);
        }
      }

      console.log(`COLLECTABLES REQUEST [${collectables.length}]`);

      return res.json(collectables);
    } else {
      throw new Error('Collectables error!');
    }
  } catch(e) {
    console.error(e);
    
    return res.status(500).send({
      error: true,
      message: "[/collectable] error!",
    });    
  }
};

module.exports = CollectableService;
