const redisClient = require("../client");
const _ = require('lodash');
const axios = require('axios');

const GetMetricService = async (req, res) => {
  try {
    const { sectionId, force } = req.params;

    const keys = await redisClient
      .scanAsync("0", "MATCH", `get-metric-*`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "GET_METRIC ERROR", e);

        return false;
      });

    if (keys) {
      let checkMetrics = [];

      await Promise.all(
        _.map(keys[1], async (k) => {
          const rawData = await redisClient.getAsync(k);

          if (rawData) {
            const data = JSON.parse(rawData);

            if (!sectionId || (sectionId && data.sectionId === sectionId)) {
              checkMetrics.push(data);
            }
          }
        })
      );

      let params = {
        sectionId,
        itemsPerPage: 0,
        properties: ['id', 'sectionId', 'name', 'category', 'image', 'orderBy'],
      }

      const total = await axios({
        method: "get",
        url: `${process.env.API_HOST}/check_metrics`,
        params,
        headers: { "X-Username": "agrocacher [GET GET_METRIC]" },
      })
        .then((response) => response.data)
        .then((data) => data["hydra:totalItems"]);      

      if (checkMetrics.length < total || force === "1") {
        checkMetrics = [];

        // FIXME: Partial loading might be better
        const dbCheckMetrics = await axios({
          method: "get",
          url: `${process.env.API_HOST}/check_metrics`,
          params: {
            ...params,
            itemsPerPage: 2000,
          },
          headers: { 
            "X-Username": "agrocacher [GET GET_METRIC]", 
            Accept: "application/json",
            "Content-Type": "application/json", 
          },
        })
          .then((response) => response.data)
          // .then((data) => data["hydra:member"]);

        for (const checkMetric of dbCheckMetrics) {
          const k = `get-metric-${checkMetric.sectionId}-${checkMetric.id}`;

          checkMetric.metricType = checkMetric.category === 'Томаты' && 'tomato' || checkMetric.category === 'Огурцы' && 'cucumber' || 'unknown';

          checkMetrics.push(checkMetric);

          await redisClient.setAsync(k, JSON.stringify(checkMetric));
          await redisClient.expireAsync(k, 3600);
        }
      }

      console.log(`GET_METRICS REQUEST [${checkMetrics.length}]`);

      return res.json(checkMetrics);
    } else {
      throw new Error('GET_METRICS ERROR!');
    }
  } catch(e) {
    console.error(e);
    
    return res.status(500).send({
      error: true,
      message: "[/get-metric] error!",
    });    
  }
};

module.exports = GetMetricService;
