const redisClient = require("../client");
const _ = require("lodash");
const axios = require("axios");

module.exports = IntensityDescriptionService = async (req, res) => {
  try {
    const { force } = req.params;

    const keys = await redisClient
      .scanAsync("0", "MATCH", `intensity-description-*`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "INTENSITY_DESCRIPTION ERROR", e);

        return false;
      });

    if (keys) {
      let intensityDescriptions = [];

      await Promise.all(
        _.map(keys[1], async (k) => {
          const rawData = await redisClient.getAsync(k);

          if (rawData) {
            intensityDescriptions[k] = JSON.parse(rawData);
          }
        })
      );

      if (_.size(intensityDescriptions) <= 0 || force === "1") {
        // always invalidate
        intensityDescriptions = await axios({
          method: "get",
          url: `${process.env.API_HOST}/intensity_descriptions?itemsPerPage=1000`,
          headers: { "X-Username": "agrocacher [GET INTENSITY_DESCRIPTIONS]" },
        })
          .then((response) => response.data)
          .then((data) => data["hydra:member"]);

        for (const description of intensityDescriptions) {
          const k = `intensity-description-${description.id}`;

          await redisClient.setAsync(k, JSON.stringify(description));
          await redisClient.expireAsync(k, 3600);
        }
      }

      console.log(
        `INTENSITY_DESCRIPTIONS REQUEST [${_.size(intensityDescriptions)}]`
      );

      return res.json(intensityDescriptions);
    } else {
      return res.status(500).send({
        error: true,
        message: "Intensity-descriptions error!",
      });
    }
  } catch(e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: "[/intensityDescription] error!",
    });    
  }
};
