const redisClient = require("../client");
const axios = require('axios');
const _ = require('lodash');

const IntensityService = async (req, res) => {
  try {
    const { force } = req.params;

    const keys = await redisClient
      .scanAsync("0", "MATCH", `intensity-entity-*`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "INTENSITY ERROR", e);

        return false;
      });

    if (keys) {
      let intensities = [];

      await Promise.all(
        _.map(keys[1], async (k) => {
          const rawData = await redisClient.getAsync(k);

          if (rawData) {
            intensities.push(JSON.parse(rawData));
          }
        })
      );

      if (intensities.length <= 0 || force === "1") {
        intensities = await axios({
          method: "get",
          url: `${process.env.API_HOST}/intensities?itemsPerPage=100`,
          headers: { "X-Username": "agrocacher [GET INTENSITIES]" },
        })
          .then((response) => response.data)
          .then((data) => data["hydra:member"]);

        for (const intensity of intensities) {
          const k = `intensity-entity-${intensity.id}`;

          await redisClient.setAsync(k, JSON.stringify(intensity));
          await redisClient.expireAsync(k, 3600);
        }
      }

      console.log(`INTENSITIES REQUEST [${intensities.length}]`);

      return res.json(intensities);
    } else {
      throw new Error('Intensities error!');
    }
  } catch(e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: "[/intensity] error!",
    });    
  } 
};

module.exports = IntensityService;
