const redisClient = require("../client");
const _ = require("lodash");
const axios = require("axios");

module.exports = PestService = async (req, res) => {
  try {
    const { type, force } = req.params;

    const keys = await redisClient
      .scanAsync("0", "MATCH", `pest-*`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "PEST ERROR", e);

        return false;
      });

    if (keys) {
      let pests = [];

      await Promise.all(
        _.map(keys[1], async (k) => {
          const rawData = await redisClient.getAsync(k);

          if (rawData) {
            const data = JSON.parse(rawData);
            if (!type || (type && data.type === type)) {
              pests.push(data);
            }
          }
        })
      );

      let query = type && `type=${type}` || '';

      const total = await axios({
        method: "get",
        url: `${process.env.API_HOST}/pests?${query}&itemsPerPage=0`,
        headers: { "X-Username": "agrocacher [GET PESTS]" },
      })
        .then((response) => response.data)
        .then((data) => data["hydra:totalItems"]);

      if (pests.length < total || force === "1") {
        pests = await axios({
          method: "get",
          url: `${process.env.API_HOST}/pests?${query}&itemsPerPage=1000`,
          headers: { "X-Username": "agrocacher [GET PESTS]" },
        })
          .then((response) => response.data)
          .then((data) => data["hydra:member"]);

        for (const pest of pests) {
          const k = `pest-${pest.id}`;

          await redisClient.setAsync(k, JSON.stringify(pest));
          await redisClient.expireAsync(k, 3600);
        }
      }

      console.log(`PESTS REQUEST [${pests.length}]`);

      return res.json(pests);
    } else {
      return res.status(500).send({
        error: true,
        message: "Pests error!",
      });
    }
  } catch(e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: "[/pest] error!",
    });    
  }
};
