const redisClient = require("../client");
const axios = require('axios');
const _ = require('lodash');

const PlantService = async (req, res) => {
  try {
    const { greenhouseId, force } = req.params;

    const keys = await redisClient
      .scanAsync("0", "MATCH", `plant-*`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "PLANT ERROR", e);

        return false;
      });

    if (keys) {
      let plants = [];

      await Promise.all(
        _.map(keys[1], async (k) => {
          const rawData = await redisClient.getAsync(k);

          if (rawData) {
            const data = JSON.parse(rawData);

            if (!greenhouseId || (greenhouseId && data.greenhouseId === greenhouseId)) {
              plants.push(data);
            }
          }
        })
      );

      let query = greenhouseId && `row.greenhouseId=${greenhouseId}` || '';

      const total = await axios({
        method: "get",
        url: `${process.env.API_HOST}/plant_phenologies?${query}&itemsPerPage=0`,
        headers: { "X-Username": "agrocacher [GET PLANTS]" },
      })
        .then((response) => response.data)
        .then((data) => data["hydra:totalItems"]);

      if (plants.length < total || force === "1") {
        plants = [];

        // FIXME: Partial loading might be better
        const dbPlants = await axios({
          method: "get",
          url: `${process.env.API_HOST}/plant_phenologies?${query}&itemsPerPage=10000`,
          headers: { 
            "X-Username": "agrocacher [GET PLANTS]", 
            Accept: "application/json",
            "Content-Type": "application/json", 
          },
        })
          .then((response) => response.data);
          //.then((data) => data["hydra:member"]);

        for (const plant of dbPlants) {
          const k = `plant-${plant.row.greenhouseId}-${plant.id}`;

          const p = {
            id: plant.id,
            rowId: plant.rowId,
            rowNumber: plant.row.rowNumber,
            orderBy: plant.row.orderBy,
            greenhouseId: plant.row.greenhouseId,
            type: plant.row.type,
            name: plant.name,
          }

          plants.push(p);

          await redisClient.setAsync(k, JSON.stringify(p));
          await redisClient.expireAsync(k, 3600);
        }
      }

      console.log(`PLANTS REQUEST [${plants.length}]`);

      return res.json(plants);
    } else {
      throw new Error('Plants error!');
    }
  } catch(e) {
    console.error(e);
    
    return res.status(500).send({
      error: true,
      message: "[/plant] error!",
    });    
  }
};

module.exports = PlantService;
