const axios = require('axios');
const moment = require('moment');
const _ = require('lodash');

const SetPriorityService = async (req, res) => {
  try {
    const { workerId, write } = req.params;

    if (!workerId) {
      throw new Error('[/set-priority] MISSING WORKER ID');
    }

    const worker = await axios({
      url: `${process.env.API_HOST}/workers/${workerId}`,
      method: 'get',
      headers: {
        'X-Username': 'agrocacher [GET PRIORITY WORKER]',
        Accept: 'application/json',
        'Content-Type': 'application/json',
      },
    })
      .then((res) => res.data);

    if(!worker.isActive) {
      throw new Error('[/set-priority] INACTIVE WORKER');
    }

    const ratings = _.keyBy(await axios({
      url: `${process.env.API_HOST}/ratings?workerId=${workerId}&itemsPerPage=100`,
      method: 'get',
      headers: {
        'X-Username': 'agrocacher [GET PRIORITY RATINGS]',
        Accept: 'application/json',
        'Content-Type': 'application/json',
      },
    })
      .then((res) => res.data), 'sectionId');

    const priorities = _.keyBy(await axios({
      url: `${process.env.API_HOST}/check_priorities?workerId=${workerId}&itemsPerPage=100`,
      method: 'get',
      headers: {
        'X-Username': 'agrocacher [GET PRIORITY PRIORITIES]',
        Accept: 'application/json',
        'Content-Type': 'application/json',
      },
    })
      .then((res) => res.data), 'sectionId');

    const output = _.map(ratings, r => ({
      sectionId: r.sectionId,
      priorityId: null,
      name: r.name,
      dynamicRating: r.dynamicRating,
      rating: r.ratingInt,
      oldPriority: null,
      newPriority: null,
      reason: 'n/a',
      saved: false,
    }));

    for(const out of output) {
      const createdAt = moment(worker.createdAt);
      //const now = moment(createdAt).add(3, 'day');
      const now = moment();

      const diffImport = now.diff(createdAt, 'days');

      const priority = priorities[out.sectionId] || {};

      out.priorityId = priority.id || null;
      
      if(diffImport >= 3 && diffImport <= 7) {
        out.oldPriority = priority.priority || null,
        out.newPriority = 4;
        out.reason = 'NEW WORKER';
      } else if(!priority.lastCheck || now.diff(priority.lastCheck, 'days') > 14) {
        out.oldPriority = priority.priority || null,
        out.newPriority = 3;
        out.reason = 'CHECK EXPIRATION';
      } else if(out.dynamicRating <= 3) {
        out.oldPriority = priority.priority || null,
        out.newPriority = 2;
        out.reason = 'LOW_PRIORITY';
      } else {
        out.oldPriority = priority.priority || null,
        out.newPriority = 1;
        out.reason = 'DEFAULT PRIORITY';        
      }
    }

    if(write === '1') {
      for(const out of output) {
        if(out.oldPriority != out.newPriority) {
          if(_.isNull(out.priorityId)) {
            await axios({
              method: "post",
              url: `${process.env.API_HOST}/check_priorities`,
              headers: {
                "X-Username": "agrocacher [POST CHECK_PRIORITY]",
                Accept: "application/ld+json",
                "Content-Type": "application/ld+json",
              },
              data: {
                section: `sections/${out.sectionId}`,
                worker: `workers/${workerId}`,
                priority: out.newPriority,
                lastCheck: moment().format(), // FIXME: Should be null instead
              },
            });
          } else {
            await axios({
              method: "patch",
              url: `${process.env.API_HOST}/check_priorities/${out.priorityId}`,
              headers: {
                "X-Username": "agrocacher [PATCH CHECK_PRIORITY]",
                'Content-Type': 'application/merge-patch+json',
              },
              data: {
                priority: out.newPriority,
              },
            });
          }

          out.saved = true;
        }
      }
    }

    return res.json({
      error: false,
      name: worker.name,
      greenhouse: worker.greenhouse.description,
      workerId: workerId || null,
      importDate: worker.createdAt,
      currentDate: moment().format(),
      output,
    });
  } catch (e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: `[/set-priority error] [${e}]`,
    });
  }
};

module.exports = SetPriorityService;
