const axios = require("axios");
const moment = require("moment");
const _ = require("lodash");
const getRequest = require("../helpers/request");

const SetRatingService = async (req, res) => {
  try {
    const { workerId, write } = req.params;

    if (!workerId) {
      throw new Error("[/set-rating] MISSING WORKER ID");
    }

    const worker = await axios({
      url: `${process.env.API_HOST}/workers/${workerId}`,
      method: "get",
      headers: {
        "X-Username": "agrocacher [GET PRIORITY WORKER]",
        Accept: "application/json",
        "Content-Type": "application/json",
      },
    })
      .then((res) => res.data);

    if (!worker.isActive) {
      throw new Error("[/set-rating] INACTIVE WORKER");
    }

    const ratings = _.keyBy(await axios({
      url: `${process.env.API_HOST}/ratings?workerId=${workerId}&itemsPerPage=100`,
      method: "get",
      headers: {
        "X-Username": "agrocacher [GET PRIORITY RATINGS]",
        Accept: "application/json",
        "Content-Type": "application/json",
      },
    })
      .then((res) => res.data), "sectionId");

    const output = _.map(ratings, (r) => {
      const ratingInt = Math.round(r.dynamicRating) * 2 || 1;
      const rating = (ratingInt - 1) / 10;
      return {
        sectionId: r.sectionId,
        ratingId: r.id,
        name: r.name,
        oldRating: r.rating,
        oldRatingInt: r.ratingInt,
        dynamicRating: r.dynamicRating,
        newRating: Number(rating.toFixed(1)),
        newRatingInt: ratingInt,
        ratingUpdatedAt: r.ratingUpdatedAt,
        reason: "n/a",
        saved: false,
      };
    });

    const freqRatingResponse = await getRequest("configurations", {
      id: "freqRating",
    });
    if (freqRatingResponse["@type"] === "hydra:Error") {
      throw new Error(freqRatingResponse);
    }
    const { numberInMounth } = freqRatingResponse["hydra:member"][0].config;
    const threshold = moment(new Date()).subtract(Number(numberInMounth), "month");

    if (write === "1") {
      for (const out of output) {
        const isNeedWriteRating = threshold.isAfter(moment(out.ratingUpdatedAt));
        if (isNeedWriteRating) {
          // обнуление записи для ratingUpdatedAt в случае одинакового рейтинга
          await axios({
            method: "patch",
            url: `${process.env.API_HOST}/ratings/${out.ratingId}`,
            headers: {
              "X-Username": "agrocacher [PATCH RATINGS]",
              "Content-Type": "application/merge-patch+json",
            },
            data: {
              rating: 0,
              ratingInt: 0,
            },
          });
          const rating = await axios({
            method: "patch",
            url: `${process.env.API_HOST}/ratings/${out.ratingId}`,
            headers: {
              "X-Username": "agrocacher [PATCH RATINGS]",
              "Content-Type": "application/merge-patch+json",
            },
            data: {
              rating: out.newRating,
              ratingInt: out.newRatingInt,
            },
          });
          out.ratingUpdatedAt = rating.data.ratingUpdatedAt;
          out.saved = true;
        }
      }
    }

    return res.json({
      error: false,
      name: worker.name,
      greenhouse: worker.greenhouse.description,
      workerId: workerId || null,
      importDate: worker.createdAt,
      currentDate: moment().format(),
      output,
    });
  } catch (e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: `[/set-rating error] [${e}]`,
    });
  }
};

module.exports = SetRatingService;
