const redisClient = require("../client");
const axios = require("axios");
const moment = require("moment");
const fs = require("fs");
const _ = require('lodash');
const globalApkData = require("../apk");

module.exports = StatusService = async (req, res) => {
  try {
    const {workerId, force} = req.params;

    const workingTimeConfigKey = workerId
      ? `workingTimeConfig-${workerId}`
      : "workingTimeConfig";
    const apkDataKey = workerId ? `apkData-${workerId}` : "apkData";
    const workerKey = null;
    //const workerKey = workerId ? `worker-${workerId}` : null;

    let workingTimeConfig = null;
    let apkData = null;
    let worker = {};

    const rawWorkingTimeConfig = await redisClient
      .getAsync(workingTimeConfigKey)
      .catch((e) => {
        console.log(
          "ERROR",
          `GET_WORKING_TIME_CONFIG [${workerId || "anonymous"}] ERROR`,
          e
        );

        return false;
      });

    if (rawWorkingTimeConfig) {
      workingTimeConfig = JSON.parse(rawWorkingTimeConfig);
    }

    const rawApkData = await redisClient.getAsync(apkDataKey).catch((e) => {
      console.log(
        "ERROR",
        `GET_APK_DATA [${workerId || "anonymous"}] ERROR`,
        e
      );

      return false;
    });

    if (rawApkData) {
      apkData = JSON.parse(rawApkData);
    }

    if (workerKey) {
      const rawWorker = await redisClient.getAsync(workerKey).catch((e) => {
        console.log(
          "ERROR",
          `GET_WORKER [${workerId || "anonymous"}] ERROR`,
          e
        );

        return false;
      });

      if (rawWorker) {
        worker = JSON.parse(rawWorker);
      }
    }

    if (!workingTimeConfig || force === "1") {
      workingTimeConfig = await axios({
        method: "get",
        url: `${process.env.API_HOST}/configurations/workingTimeConfig`,
        headers: {
          "X-Username": "agrocacher [GET CONFIGURATION]",
          Accept: "application/ld+json",
          "Content-Type": "application/ld+json",
        },
      })
        .then((r) => r.data)
        .then(
          (data) =>
            data.config || {
              start: "00:00",
              finish: "23.59",
            }
        )
        .then((config) => {
          redisClient.setAsync(workingTimeConfigKey, JSON.stringify(config));
          redisClient.expireAsync(workingTimeConfigKey, 3600);

          return config;
        })
        .catch((e) => {
          console.error("CONFIG GET ERROR", e);
        });
    }

    const timeFrom = moment(workingTimeConfig.start, "HH:mm");
    const timeTo = moment(workingTimeConfig.finish, "HH:mm");

    if (!apkData || force === "1") {
      apkData = {...globalApkData};

      redisClient.setAsync(apkDataKey, JSON.stringify(apkData));
      redisClient.expireAsync(apkDataKey, 3600);
    }

    if (workerKey && (_.isEmpty(worker) || force === "1")) {
      worker = await axios({
        url: `${process.env.API_HOST}/workers/${workerId}`,
        method: "get",
        params: {
          groups: ["assign"],
          properties: [
            "id",
            "name",
            "code",
            "isActive",
            "spreadTill",
            "nfcMode",
          ],
        },
        headers: {
          "X-Username": "agrocacher [GET STATUS WORKER]",
          Accept: "application/json",
          "Content-Type": "application/json",
        },
      }).then((res) => res.data);

      redisClient.setAsync(workerKey, JSON.stringify(worker));
      redisClient.expireAsync(workerKey, 60);
    }

    return res.send({
      error: false,
      message: "WELCOME!",
      environment: process.env.APP_ENV,
      timestamp: moment().format(),
      workingDayFrom: moment()
        .set({
          hour: timeFrom.get("hour"),
          minute: timeFrom.get("minute"),
          second: timeFrom.get("second"),
        })
        .format(),
      workingDayTo: moment()
        .set({
          hour: timeTo.get("hour"),
          minute: timeTo.get("minute"),
          second: timeTo.get("second"),
        })
        .format(),
      versionCode: apkData.versionCode,
      versionName: apkData.versionName,
      versionUrl: `${process.env.APK_HOST}/beta/${apkData.versionFolder}/app.apk`,
      updateRequired: false,
      workerId: workerId || null,
      // spreadTill: worker.spreadTill || null,
    });
  } catch (e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: "Status error!",
    });
  }
};
