const redisClient = require("../client");
const moment = require('moment');
const { v4: uuidv4 } = require("uuid");

const StoreEventService = async (req, res) => {
  try {
    if (!req.body.payload) {
      console.log("ERROR", "PAYLOAD MISSING", req.body);

      return res.status(400).send({
        error: true,
        message: "Payload missing!",
      });
    }

    const expire = req.body.expire;
    const value = req.body.payload;

    value.registeredAt = moment().format();
    value.id = uuidv4();

    const key = `event-${req.body.key}-${moment().valueOf()}`;

    await redisClient.setAsync(key, JSON.stringify(value));
    if (expire) {
      await redisClient.expireAsync(key, expire);
    }

    console.info(`KEY CREATED [${key}] WITH EXPIRE [${expire}]`);

    return res.send({
      error: false,
      message: "SUCCESS!",
    });
  } catch(e) {
    console.error(e);
    
    return res.status(500).send({
      error: true,
      message: "[/store-event] error!",
    });    
  }  
};

module.exports = StoreEventService;
