const redisClient = require("../client");
const axios = require('axios');
const moment = require('moment');
const _ = require('lodash');

const StoreTaskService = async (req, res) => {
  try {
    if (!req.body.payload) {
      console.log("ERROR", "PAYLOAD MISSING", req.body);

      return res.status(400).send({
        error: true,
        message: "Payload missing!",
      });
    }

    const taskId = req.body.payload.id;
    const workerId = req.body.payload.workerId;
    const previousWorkerId = req.body.payload.previousWorkerId;
    const expire = req.body.expire;
    const value = req.body.payload;

    const key = `${workerId}-${taskId}`;

    if(workerId) {
      await redisClient.delAsync(`task-${workerId}`);
    }

    if(previousWorkerId) {
      await redisClient.delAsync(`task-${previousWorkerId}`);
    }

    const keys = await redisClient
      .scanAsync("0", "MATCH", `*-${taskId}`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "TASKS DEL ERROR", e);

        return false;
      });

    if (keys) {
      await Promise.all(
        _.map(keys[1], async (k) => {
          if (k !== key) {
            // await redisClient.delAsync(k);
            await redisClient.setAsync(k, JSON.stringify(value));
            if (expire) {
              await redisClient.expireAsync(k, expire);
            }

            console.info(`KEY UPDATED [${k}] WITH EXPIRE [${expire}]`);
          }
        })
      );
    } else {
      return res.status(500).send({
        error: true,
        message: "Tasks del error!",
      });
    }

    if (!taskId || !workerId) {
      console.log("ERROR", `COMPOSITE KEY MISSING [${workerId}-${taskId}]`);

      return res.status(400).send({
        error: true,
        message: "Payload key missing!",
      });
    }

    const rawTask = await redisClient.getAsync(key);

    if (!rawTask) {
      /*
      await axios({
        method: "post",
        url: `${process.env.API_HOST}/task_metrics`,
        headers: {
          "X-Username": "agrocacher [POST TASK_METRIC]",
          Accept: "application/ld+json",
          "Content-Type": "application/ld+json",
        },
        data: {
          registeredAt: moment().format(),
          task: `tasks/${taskId}`,
          worker: `workers/${workerId}`,
          type: "cache-placing",
        },
      }).catch((e) => {
        console.error("TASK METRICS ERROR", e);
      });
      */
    }

    await redisClient.setAsync(key, JSON.stringify(value));
    if (expire) {
      await redisClient.expireAsync(key, expire);
    }

    console.info(`KEY CREATED [${key}] WITH EXPIRE [${expire}]`);

    return res.send({
      error: false,
      message: "SUCCESS!",
    });
  } catch(e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: "[/store-task] error!",
    });    
  }  
};

module.exports = StoreTaskService;
