const redisClient = require("../client");
const axios = require('axios');
const moment = require('moment');
const _ = require('lodash');

const TaskHistoryService = async (req, res) => {
  try {
    const { workerId, force } = req.params;
    const key = `task-history-${workerId}`;

    let tasks = [];

    const rawData = await redisClient
      .getAsync(`task-history-${workerId}`)
      .catch((e) => {
        console.log("ERROR", "GET_TASK_HISTORY ERROR", e);

        return false;
      });

    if (rawData) {
      tasks = JSON.parse(rawData);
    }

    if (tasks) {
      // empty array || filled

      if (tasks.length <= 0 || force === "1") {
        const dayOfMonth = moment().date();

        let startDate = moment().startOf('month');
        let finishDate = moment().endOf('month');

        if(dayOfMonth >= 16) {
          startDate.date(16);
        } else {
          finishDate.date(15);
        }

        tasks = await axios({
          method: "get",
          url: `${process.env.API_HOST}/tasks`,
          params: {
            workerId,
            itemsPerPage: 30,
            "order[finishedAt]": "desc",
            isFinished: true,
            "finishedAt[before]": finishDate.format(),
            "finishedAt[after]": startDate.format(),
            groups: [
              'mobile',
            ],
            properties: [
              'id',
              'name',
              'workerId',
              'workerName',
              'sectionName',
              'rowNumber',
              'isFinished',
              'startedAt',
              'finishedAt',
              'startAt',
              'finishAt',
              'orderBy',
              'isActive',
              'department',
              'isHarvesting',
              'taskLimit',
              'shelfAmount',
              'rowType',
              'isGuarding',
              'isPhenology',
              'qualityTaskId',
              'qualityTask',
              'isSpraying',
              'isQuality',
            ],
          },
          headers: { "X-Username": "agrocacher [GET GET_TASK_HISTORY]" },
        })
          .then((response) => response.data)
          .then((data) => data["hydra:member"]);

        await redisClient.setAsync(key, JSON.stringify(tasks));
        await redisClient.expireAsync(key, 3600);
      }

      console.log(`GET_TASK_HISTORY REQUEST [${tasks.length}]`);

      return res.json(tasks);
    } else {
      throw new Error('task-history error!');
    }
  } catch(e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: "[/task-history] error!",
    });    
  }
};

module.exports = TaskHistoryService;
