const redisClient = require("../client");
const axios = require('axios');
const moment = require('moment');
const _ = require('lodash');

const COLLECTION_TASKS = 'tasks';
const COLLECTION_CONFIGURATIONS = 'configurations'; // TODO: Check working time

const TaskService = async (req, res) => {
  try {
    const { workerId, force } = req.params;
    const key = `task-${workerId}`;

    if (!workerId) {
      throw new Error('[GET_TASK_SERVICE] MISSING WORKER ID');
    }

    let tasks = [];

    const rawData = await redisClient
      .getAsync(key)
      .catch((e) => {
        console.log("ERROR", "[GET_TASK_SERVICE] ERROR", e);

        return false;
      });

    if (rawData) {
      tasks = JSON.parse(rawData);
    }

    if (tasks) {
      if (tasks.length <= 0 || force === "1") {
        const params = {
          itemsPerPage: 30,
          workerId,
          isFinished: false,
          isActive: true,
          startAt: {
            before: moment().format(),
          },
          finishAt: {
            after: moment().format(),
          },
          groups: [
            'mobile',
          ],
          properties: [
            'id',
            'name',
            'workerId',
            'workerName',
            'sectionName',
            'rowNumber',
            'isFinished',
            'startedAt',
            'finishedAt',
            'startAt',
            'finishAt',
            'orderBy',
            'isActive',
            'department',
            'isHarvesting',
            'taskLimit',
            'shelfAmount',
            'rowType',
            'isGuarding',
            'isPhenology',
            'qualityTaskId',
            'qualityTask',
            'isSpraying',
            'isQuality',
          ],
        };

        tasks = await axios({
          method: "get",
          url: `${process.env.API_HOST}/${COLLECTION_TASKS}`,
          params,
          headers: { "X-Username": "agrocacher [GET GET_TASK_SERVICE]" },
        })
          .then((response) => response.data)
          .then((data) => data["hydra:member"]);

        await redisClient.setAsync(key, JSON.stringify(tasks));
        await redisClient.expireAsync(key, 3600);          
      }

      console.log(`[GET_TASK_SERVICE] REQUEST [${tasks.length}]`);

      return res.json(tasks);
    } else {
      throw new Error('[GET_TASK_SERVICE] REDIS ERROR');
    }
  } catch (e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: `TASKS SERVICE GET ERROR! [${e}]`,
    });
  }
};

module.exports = TaskService;
