const redisClient = require("../client");
const axios = require('axios');
const moment = require('moment');
const _ = require('lodash');

const TrolleyHistoryService = async (req, res) => {
  try {
    const { workerId, force } = req.params;
    const key = `trolley-history-${workerId}`;

    let trolleys = [];

    const rawData = await redisClient
      .getAsync(`trolley-history-${workerId}`)
      .catch((e) => {
        console.log("ERROR", "GET_TROLLEY_HISTORY ERROR", e);

        return false;
      });

    if (rawData) {
      trolleys = JSON.parse(rawData);
    }

    if (trolleys) {
      // empty array || filled

      if (trolleys.length <= 0 || force === "1") {
        const dayOfMonth = moment().date();

        let startDate = moment().startOf('month');
        let finishDate = moment().endOf('month');

        if(dayOfMonth >= 16) {
          startDate.date(16);
        } else {
          finishDate.date(15);
        }

        trolleys = await axios({
          method: "get",
          url: `${process.env.API_HOST}/trolleys`,
          params: {
            workerId,
            type: 'finish',
            itemsPerPage: 100,
            "order[registeredAt]": "desc",
            "registeredAt[before]": finishDate.format(),
            "registeredAt[after]": startDate.format(),
            properties: ["id", "workerId", "trolley", "registeredAt", "type"],
          },
          headers: { "X-Username": "agrocacher [GET GET_TROLLEY_HISTORY]" },
        })
          .then((response) => response.data)
          .then((data) => data["hydra:member"]);

        await redisClient.setAsync(key, JSON.stringify(trolleys));
        await redisClient.expireAsync(key, 3600);
      }

      console.log(`GET_TROLLEY_HISTORY REQUEST [${trolleys.length}]`);

      return res.json(trolleys);
    } else {
      throw new Error('trolley-history error!')
    }
  } catch(e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: "[/trolley-history] error!",
    });    
  }
};

module.exports = TrolleyHistoryService;
