const redisClient = require("../client");
const axios = require('axios');
const _ = require('lodash');

const ValveService = async (req, res) => {
  try {
    const { greenhouseId, force } = req.params;

    const keys = await redisClient
      .scanAsync("0", "MATCH", `valve-*`, "COUNT", "100000")
      .catch((e) => {
        console.log("ERROR", "VALVE ERROR", e);

        return false;
      });
    if (keys) {
      let valves = [];

      await Promise.all(
        _.map(keys[1], async (k) => {
          const rawData = await redisClient.getAsync(k);

          if (rawData) {
            const data = JSON.parse(rawData);

            if (!greenhouseId || (greenhouseId && data.greenhouseId === greenhouseId)) {
              valves.push(data);
            }
          }
        })
      );

      let query = greenhouseId && `greenhouseId=${greenhouseId}` || '';

      const total = await axios({
        method: "get",
        url: `${process.env.API_HOST}/valves?${query}&itemsPerPage=0`,
        headers: { "X-Username": "agrocacher [GET VALVES]" },
      })
      .then((response) => response.data)
      .then((data) => data["hydra:totalItems"]);

      if (valves.length < total || force === "1") {
        valves = [];

        // FIXME: Partial loading might be better
        const dbValves = await axios({
          method: "get",
          url: `${process.env.API_HOST}/valves?${query}&properties[]=id&properties[]=volume&properties[]=potentialHydrogen&properties[]=electricConduction&properties[]=workDescription&properties[]=number&properties[]=greenhouseId&itemsPerPage=10000`,
          headers: { 
            "X-Username": "agrocacher [GET VALVES]", 
            Accept: "application/json",
            "Content-Type": "application/json", 
          },
        })
        .then((response) => response.data);
        //.then((data) => data["hydra:member"]);
        const valveLimits = await axios({
          method: 'get',
          url: `${process.env.API_HOST}/valve_limits`,
          headers: {
            'X-Username': 'agrocacher [GET VALVE_LIMITS]',
            Accept: 'application/json',
            'Content-Type': 'application/json',
          },
        }).then((response) => response.data);
        for (const valve of dbValves) {

          const limits = valveLimits
            .filter(
              ({ workDescription }) => workDescription === valve.workDescription
            ).reduce(
              (acc, { measurementDescription, minLimit, maxLimit }) => ({
                ...acc,
                [`minLimit${_.upperFirst(measurementDescription)}`]: minLimit,
                [`maxLimit${_.upperFirst(measurementDescription)}`]: maxLimit,
              }),
              {}
            );

          const k = `valve-${valve.greenhouseId}-${valve.id}`;
          
          const p = {
            id: valve.id,
            greenhouseId: valve.greenhouseId,
            volume: valve.volume,
            potentialHydrogen: valve.potentialHydrogen,
            electricConduction: valve.electricConduction,
            workDescription: valve.workDescription,
            number: valve.number,
            ...limits,
          }

          valves.push(p);

          await redisClient.setAsync(k, JSON.stringify(p));
          await redisClient.expireAsync(k, 3600);
        }
      }

      console.log(`VALVES REQUEST [${valves.length}]`);

      return res.json(valves);
    } else {
      throw new Error('Valves error!');
    }
  } catch(e) {
    console.error(e);
    
    return res.status(500).send({
      error: true,
      message: "[/valve] error!",
    });    
  }
};

module.exports = ValveService;
