const redisClient = require("../client");
const axios = require("axios");
const moment = require("moment");
const fs = require("fs");
const _ = require("lodash");

module.exports = StatusService = async (req, res) => {
  try {
    const {id, force} = req.params;

    const workerKey = `worker-${id}`;

    let worker = {};

    if (workerKey) {
      const rawWorker = await redisClient.getAsync(workerKey).catch((e) => {
        console.log("ERROR", `GET_WORKER [${id}] ERROR`, e);

        return false;
      });

      if (rawWorker) {
        worker = JSON.parse(rawWorker);
      }
    }

    if (_.isEmpty(worker) || force === "1") {
      worker = await axios({
        url: `${process.env.API_HOST}/workers/${id}`,
        method: "get",
        params: {
          // groups: ["assign"],
          properties: [
            "id",
            "name",
            "code",
            "isActive",
            "spreadTill",
            "nfcMode",
            "department",
            "position",
            "departmentId",
            "positionId",
            "greenhouseId",
          ],
        },
        headers: {
          "X-Username": "agrocacher [GET WORKER]",
          Accept: "application/json",
          "Content-Type": "application/json",
        },
      })
        .then((res) => res.data)
        .catch((e) =>
          axios({
            url: `${process.env.API_HOST}/workers`,
            method: "get",
            params: {
              code: id,
	      isActive: true,
              // groups: ["assign"],
              properties: [
                "id",
                "name",
                "code",
                "isActive",
                "spreadTill",
                "nfcMode",
                "department",
                "position",
                "departmentId",
                "positionId",
                "greenhouseId",
              ],
              itemsPerPage: 1,
            },
            headers: {
              "X-Username": "agrocacher [GET WORKER]",
              Accept: "application/json",
              "Content-Type": "application/json",
            },
          })
            .then((res) => res.data)
            .then((res) => res[0])
        );

      redisClient.setAsync(`worker-${worker.code}`, JSON.stringify(worker));
      redisClient.expireAsync(`worker-${worker.code}`, 60);

      redisClient.setAsync(`worker-${worker.id}`, JSON.stringify(worker));
      redisClient.expireAsync(`worker-${worker.id}`, 60);
    }

    return res.send({
      ...worker,
    });
  } catch (e) {
    console.error(e);

    return res.status(500).send({
      error: true,
      message: "/w ERROR!",
    });
  }
};
